function [ S, window ] = STFT( signal, fftSize, shiftSize, window )
%
% Corded by D. Kitamura (d-kitamura@nii.ac.jp) on 31 Jul. 2015.
%
% This function calculates short-time Fourier transform (STFT) of input
% time-domain signal.
%
% see also
% http://d-kitamura.sakura.ne.jp/index.html
%
% [syntax]
%   [S, window] = STFT(signal, fftSize)
%   [S, window] = STFT(signal, fftSize, shiftSize)
%   [S, window] = STFT(signal, fftSize, shiftSize, window)
%
% [inputs]
%    signal: input signal (len x 1)
%   fftSize: frame length (even number, must be dividable by frsft)
% shiftSize: frame shift (default: fftSize/2)
%    window: analysis window function (frlen x 1, default: hamming window)
%
% [outputs]
%         S: spectrogram of input signal (fftSize/2+1 x frame)
%    window: used analysis window function (frlen x 1)
%

% Check errors and set default values
if (nargin < 2)
    error('Too few input arguments.');
end
channel = size(signal, 2);
if (channel ~= 1)
    error('The first augument must be a row vector.');
end
if (mod(fftSize,2) ~= 0)
    error ('The third argument must be an even number');
end
if (nargin<3)
    shiftSize = fftSize / 2;
elseif (mod(fftSize,shiftSize) ~= 0)
    error('The thrid argument must be dividable by the fourth argument.');
end
if (nargin<4)
    window = hamming_local(fftSize);
else
    windowLength = size(window, 1);
    if (windowLength ~= fftSize)
        error('The length of the second argument must be the third argument.');
    end
end

% Pad zeros
signal = [zeros(1,fftSize), signal.', zeros(1,fftSize)]';
signalLength = size(signal, 1);

% Calculate STFT
frames = floor( (signalLength - fftSize + shiftSize) / shiftSize ); % Number of frames
S = zeros( fftSize/2+1, frames );
for i = 1:frames
    sp = (i-1)*shiftSize;
    spectrum = fft( signal(sp+1:sp+fftSize,1) .* window );
    S(:,i) = spectrum(1:fftSize/2+1, 1);
end
end

%% Local function
function window = hamming_local(fftSize)
t = (1:1:fftSize).';
window = 0.54*ones(fftSize,1) - 0.46*cos(2.0*pi/fftSize*t);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% EOF %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%