function signal = invSTFT( S, shiftSize, window )
%
% Corded by D. Kitamura (d-kitamura@nii.ac.jp) on 31 Jul. 2015.
%
% This function calculates inverse short-time Fourier transform (STFT) of
% input complex spectrogram.
%
% see also
% http://d-kitamura.sakura.ne.jp/index.html
%
% [syntax]
%   signal = invSTFT(S)
%   signal = invSTFT(S, shiftSize)
%   signal = invSTFT(S, shiftSize, window)
%
% [inputs]
%          S: STFT of input signal (frlen/2+1 x frames)
%  shiftSize: frame shift (default: frlen/2)
%     window: window function used in STFT (frlen x 1, default: optimum window for hamming analysis window)
%
% [outputs]
%     signal: input signal (siglen x 1)
%

% Check errors and set default values
if (nargin < 1)
    error('Too few input arguments.');
end
[freq, frames] = size(S);
if (mod(freq,2) == 0)
    error('The number of rows of the first argument must be an odd number because it is (frame length /2)+1.');
end
fftSize = (freq-1) * 2;
if (nargin < 2)
    shiftSize = fftSize/2;
elseif (mod(fftSize,shiftSize) ~= 0)
    error('The frame length must be dividable by the second argument.');
end
if (nargin<3)
    window = hamming_local(fftSize);
end
invWindow = optSynwnd_local( window, shiftSize );

% Inverse STFT
spectrum = zeros(fftSize,1);
tmpSignal = zeros((frames-1)*shiftSize+fftSize,1);
for i = 1:frames
    sp = (i-1)*shiftSize;
    spectrum(1:fftSize/2+1,1) = S(:,i);
    spectrum(1,1) = spectrum(1,1)/2;
    spectrum(fftSize/2+1,1) = spectrum(fftSize/2+1,1)/2;
    tmpSignal(sp+1:sp+fftSize,1) = tmpSignal(sp+1:sp+fftSize,1)+real(ifft(spectrum,fftSize).*invWindow)*2;
end
signal = tmpSignal(fftSize+1:(frames-1)*shiftSize+fftSize,1);
end

%% Local function
function window = hamming_local(fftSize)
t = (1:1:fftSize).';
window = 0.54*ones(fftSize,1) - 0.46*cos(2.0*pi/fftSize*t);
end

%% Local function
function synthesizedWindow = optSynwnd_local(analysisWindow,shiftSize)
fftSize = size(analysisWindow,1);
synthesizedWindow = zeros(fftSize,1);
for i = 1:shiftSize
    amp = 0.0;
    for j = 1:fftSize/shiftSize
        amp = amp+analysisWindow(i+(j-1)*shiftSize,1)*analysisWindow(i+(j-1)*shiftSize,1);
    end
    for j = 1:fftSize/shiftSize
        synthesizedWindow(i+(j-1)*shiftSize,1) = analysisWindow(i+(j-1)*shiftSize,1)/amp;
    end
end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% EOF %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%